// hierlvw.cpp : implementation of the CHierlistView class
//

#include "stdafx.h"
#include "hierlist.h"

#include "hierldoc.h"
#include "hierlvw.h"

#include "mainfrm.h"
#include "closelev.h"

#include "ctl3d.h"

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CHierlistView

IMPLEMENT_DYNCREATE(CHierlistView, CView)

BEGIN_MESSAGE_MAP(CHierlistView, CView)
	//{{AFX_MSG_MAP(CHierlistView)
	ON_WM_CREATE()
	ON_WM_SIZE()
	ON_COMMAND(ID_CLOSEALL, OnCloseall)
	ON_COMMAND(ID_ENABLELINES, OnEnablelines)
	ON_UPDATE_COMMAND_UI(ID_ENABLELINES, OnUpdateEnablelines)
	ON_COMMAND(ID_CLOSESEL, OnClosesel)
	ON_COMMAND(ID_CHANGEFONT, OnChangefont)
	ON_COMMAND(ID_DELITEMS, OnDelitems)
	ON_WM_ERASEBKGND()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CHierlistView construction/destruction

CHierlistView::CHierlistView() 
{
	m_pFont = 0;
	LOGFONT lf;
	m_pFont = new CFont;
	::GetObject(GetStockObject(SYSTEM_FONT), sizeof(LOGFONT), &lf);
	m_pFont->CreateFontIndirect(&lf);
}

CHierlistView::~CHierlistView()
{
	delete m_pFont;
}

BOOL CHierlistView::PreCreateWindow( CREATESTRUCT& cs )
{
	cs.style &= ~(LONG)(CS_HREDRAW|CS_VREDRAW);
	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CHierlistView drawing

void CHierlistView::OnDraw(CDC* pDC)
{
	CRect r;
	GetClientRect(&r);
	pDC->SetBkColor(::GetSysColor(COLOR_BTNFACE));
	pDC->ExtTextOut(0,0,ETO_OPAQUE,&r,0,0,0);
}



/////////////////////////////////////////////////////////////////////////////
// CHierlistView diagnostics

#ifdef _DEBUG
void CHierlistView::AssertValid() const
{
	CView::AssertValid();
}

void CHierlistView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CHierlistDoc* CHierlistView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CHierlistDoc)));
	return (CHierlistDoc*) m_pDocument;
}

#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CHierlistView message handlers

int CHierlistView::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;

	CMainFrame* mf = (CMainFrame*)AfxGetApp()->m_pMainWnd;
	ASSERT(mf && mf->IsKindOf(RUNTIME_CLASS(CMainFrame)));

	m_TestHierList.AttachResources( &mf->m_ListBoxRes1 );
	m_TestListEx.AttachResources( &mf->m_ListBoxRes1 );
	
	if( m_TestHierList.Create(
		WS_CHILD|WS_VISIBLE|WS_BORDER|/*LBS_MULTIPLESEL|LBS_EXTENDEDSEL|*/ \
		LBS_NOTIFY|LBS_NOINTEGRALHEIGHT|WS_VSCROLL|LBS_OWNERDRAWVARIABLE,\
		CRect(0,0,0,0), this, 1 ) == -1 )
		return -1;

	if( m_TestListEx.Create(
		WS_CHILD|WS_VISIBLE|WS_BORDER|LBS_MULTIPLESEL|LBS_EXTENDEDSEL| \
		LBS_NOTIFY|LBS_NOINTEGRALHEIGHT|WS_VSCROLL|LBS_OWNERDRAWVARIABLE,\
		CRect(0,0,0,0), this, 1 ) == -1 )
		return -1;

	Ctl3dSubclassCtl(m_TestHierList.m_hWnd);
	Ctl3dSubclassCtl(m_TestListEx.m_hWnd);
		
	return 0;
}

void CHierlistView::OnSize(UINT nType, int cx, int cy)
{
	const int border = 4;
	
	CView::OnSize(nType, cx, cy);
	HDWP h = ::BeginDeferWindowPos(2);
	m_TestHierList.MoveWindow( border, border, cx/2-border,cy-2*border );
	m_TestListEx.MoveWindow( cx/2+border, border, cx/2-2*border, cy-2*border );
	::EndDeferWindowPos(h);
}

void CHierlistView::OnCloseall()
{
	m_TestHierList.CloseAll();
}

void CHierlistView::OnEnablelines()
{
	m_TestHierList.EnableLines( !m_TestHierList.LinesEnabled() );	
}

void CHierlistView::OnUpdateEnablelines(CCmdUI* pCmdUI)
{
	pCmdUI->SetCheck( m_TestHierList.LinesEnabled() );	
}

void CHierlistView::OnClosesel()
{
	CCloseLevel dlg(this);
	dlg.m_Level = 1;
	if( dlg.DoModal() == IDOK )
		m_TestHierList.CloseToLevel(dlg.m_Level);
}

void CHierlistView::OnChangefont()
{
	ASSERT(m_pFont);
	if(!m_pFont)
		return;
		
	// get current font description
	LOGFONT lf;
	m_pFont->GetObject(sizeof(LOGFONT), &lf);
	
	CFontDialog dlg(&lf, CF_SCREENFONTS|CF_INITTOLOGFONTSTRUCT);
	if (dlg.DoModal() == IDOK)
	{
		// switch to new font.
		m_pFont->DeleteObject();
		if(m_pFont->CreateFontIndirect(&lf))
		{
			m_TestHierList.ChangeFont(m_pFont);
			m_TestListEx.ChangeFont(m_pFont);
		}
	}
}

void CHierlistView::OnDelitems()
{
	m_TestHierList.DeleteSelected();	
}

BOOL CHierlistView::OnEraseBkgnd(CDC* pDC)
{
	return TRUE;
}
