// TestHierList.cpp : implementation file
//

#include "stdafx.h"
#include "hierlist.h"

#if !defined(TESTHIER)
 #include "testhier.h"
#endif

#include "strstream.h"
#include "iomanip.h"

#include "dragdrop.h"

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CTestHierList

CTestHierList::CTestHierList()
{
}

CTestHierList::~CTestHierList()
{
}


BEGIN_MESSAGE_MAP(CTestHierList, CHierListBox)
	//{{AFX_MSG_MAP(CTestHierList)
	ON_WM_CREATE()
	ON_REGISTERED_MESSAGE(WM_DROP,OnDrop)
	ON_REGISTERED_MESSAGE(WM_CANDROP,OnCanDrop)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTestHierList message handlers

void CTestHierList::DrawItemEx( CListBoxExDrawStruct& ds )
{
	// All that needs to be done is to BitBlt the correct bitmap from the
	// resources at the start of ds.m_Rect and then use TextOut to draw
	// the text.  The background colour and indents are all taken care of.
	
	CHierListNode* node = (CHierListNode*)ds.m_ItemData;
	ASSERT(node);
	ASSERT(node->IsKindOf(RUNTIME_CLASS(CHierListNode)));
	
	CDC* pBmpDC  = (CDC*)&ds.m_pResources->DcBitMap(); // naughty cast - should BitBlt take a const CDC* ?
	int bmh = ds.m_pResources->BitmapHeight();
	int bmw = ds.m_pResources->BitmapWidth();

	// just for testing
	char buf[16];
	ostrstream o(buf,sizeof(buf));
	char* p = (node->m_ItemData) ? "Inserted " : "Hier ";
	o << p << node->m_Level << ends;
	
	// select bitmap from resource

	int bm_h = (ds.m_Sel)?0:bmh;
	int bm_w = (node->m_bOpen)?(bmw*2):0; // + ....
	ds.m_pDC->BitBlt( ds.m_Rect.left+1, ds.m_Rect.top, bmw, bmh, pBmpDC, bm_w, bm_h, SRCCOPY );

	ds.m_pDC->TextOut(ds.m_Rect.left+bmw+3,ds.m_Rect.top,buf);
}

BOOL CTestHierList::OpenNode( const CHierListNode* parent, int& index )
{
	int plevel = parent->m_Level;
	if( plevel<3 )
	{
		SetRedraw(FALSE);

		int parentIndex = index;

		InsertNode( 0, parent, ++index );
		InsertNode( 0, parent, ++index );
		InsertNode( 0, parent, ++index );
		InsertNode( 0, parent, ++index );
		InsertNode( 0, parent, ++index );
		int lastChild = InsertNode( 0, parent, ++index );

		index = lastChild;
								
		SetRedraw(TRUE);
		Invalidate();
		return TRUE;
	}
	else
	{
		AfxMessageBox( "Do something" );
		SetFocus();
		return FALSE;
	}	
}

void CTestHierList::DeleteSelected()
{
	// single selection so
	int sel = GetCurSel();
	if(sel!=LB_ERR)
		DeleteString(sel);
}

int CTestHierList::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CHierListBox::OnCreate(lpCreateStruct) == -1)
		return -1;

	Initialise();
	
	return 0;
}

void CTestHierList::Initialise()
{
	// no LBS_SORT for heirarchical listboxes ?
	ASSERT( !(GetStyle() & LBS_SORT) );
	
	// test nodes - data should be added from derived class
	// derived class must maintain storage for objects.	
	AfxGetApp()->BeginWaitCursor();
		
	int index = 0;
	for( int i=0; i<20 && index>=0; i++ )
	{
		index = InsertNode( 0, 0 ); 

		if( index < 0 )
			AfxMessageBox( "Too many items" );
	}
	
	AfxGetApp()->EndWaitCursor();
}

void CTestHierList::DialogInit()
{
	// This seems to work ok but are then any side effects?
	OnCreate(NULL);
}

LRESULT CTestHierList::OnCanDrop( WPARAM, LPARAM )
{
	return LRESULT(TRUE);
}

LRESULT CTestHierList::OnDrop( WPARAM, LPARAM dragData )
{
	CDragStruct* pDrag = (CDragStruct*)dragData;
	
	ASSERT(pDrag->IsKindOf(RUNTIME_CLASS(CDragStruct)));
	
	int dropIndex = ItemFromPoint(pDrag->m_ClientPt);
	if(dropIndex!=LB_ERR)
	{
		long item = GetItemData(dropIndex);
		if(item!=LB_ERR)
		{
			CHierListNode* node = (CHierListNode*)item;
			BOOL bOpen = node->m_bOpen;
			for(int i=0; i<int(pDrag->m_DragItem); i++)
			{
				if(bOpen)
					InsertNode(1,node,dropIndex+i+1);
				else
					// insert new nodes after drop index - test last child 
					InsertNode(1,node->m_pParent,dropIndex+i+1);
			}
		}		
	}

	return TRUE;
}